#pragma once
#include "communication/bridge/client/i_xmlrpc_client.h"
#include <xmlrpc-c/client.h>
#define SPDLOG_ACTIVE_LEVEL SPDLOG_LEVEL_DEBUG
#include "spdlog/spdlog.h"
#include "spdlog/stopwatch.h"
#include <chrono>

namespace spin
{

    class XmlRpcClient : public IXmlRpcClient
    {
    public:
        XmlRpcClient() : xmlTransport_(xmlrpc_c::clientXmlTransport_curl::constrOpt()
            .timeout(DEFAULT_XMLRPC_TIMOUT_MS)), xmlClient_(&xmlTransport_) {}
        ~XmlRpcClient() = default;

        void setHost(const std::string &host) override
        {
            host_ = host;
        }

        std::string host() const override
        {
            return host_;
        }

        void execute(const std::string &method, xmlrpc_c::value *result) override
        {
            execute(method, xmlrpc_c::paramList(), result, std::chrono::milliseconds(DEFAULT_XMLRPC_TIMOUT_MS));
        }
        void execute(const std::string &method, const xmlrpc_c::paramList &params, xmlrpc_c::value *result) override
        {
            execute(method, params, result, std::chrono::milliseconds(DEFAULT_XMLRPC_TIMOUT_MS));
        }

        void execute(const std::string &method, const xmlrpc_c::paramList &params, xmlrpc_c::value *result, const std::chrono::milliseconds timeout) override
        {
            xmlrpc_c::rpcPtr rpc(method, params);
            xmlrpc_c::carriageParm_curl0 carriageParm(serverUrl());

            spdlog::stopwatch sw;
            sw.reset();
            rpc->call(&xmlClient_, &carriageParm);
            if (rpc->isSuccessful())
            {
                *result = rpc->getResult();
                SPDLOG_DEBUG("RPC: {} successful in {}ms", method, sw.elapsed_ms().count());
            } else
            {
                SPDLOG_WARN("RPC: {} failed in {}ms", method, sw.elapsed_ms().count());
                auto fault = rpc->getFault();
                SPDLOG_WARN("RPC: {}, faultString: {}", method, fault.getDescription());
                throw std::runtime_error("XmlRpc failed");
            }
        }

    private:
        std::string serverUrl() const
        {
            return "http://" + host_ + ":" + std::to_string(PORT) + "/RPC2";
        }
        xmlrpc_c::client_xml xmlClient_;
        xmlrpc_c::clientXmlTransport_curl xmlTransport_;
        std::string host_;
        static constexpr int PORT = 8088;
        static constexpr int DEFAULT_XMLRPC_TIMOUT_MS = 200;
    };

} // namespace spin